Package: txcutr
Title: Transcriptome CUTteR
Description: Various mRNA sequencing library preparation methods generate 
    sequencing reads specifically from the transcript ends. Analyses that focus 
    on quantification of isoform usage from such data can be aided by using 
    truncated versions of transcriptome annotations, both at the alignment or 
    pseudo-alignment stage, as well as in downstream analysis. This package 
    implements some convenience methods for readily generating such truncated 
    annotations and their corresponding sequences.
Version: 1.18.0
Date: 2025-09-07
Authors@R: 
    person(given = "Mervin",
           family = "Fansler",
           role = c("aut", "cre"),
           email = "mervin.fansler@bric.ku.dk",
           comment = c(ORCID = "0000-0002-4108-4218"))
License: GPL-3
URL: https://github.com/mfansler/txcutr
BugReports: https://github.com/mfansler/txcutr/issues
Encoding: UTF-8
Depends: R (>= 4.5.0)
Imports: AnnotationDbi, GenomicFeatures, txdbmaker, IRanges,
        GenomicRanges, BiocGenerics, Biostrings, S4Vectors,
        rtracklayer, BiocParallel, stats, methods, utils
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.1
biocViews: Alignment, Annotation, RNASeq, Sequencing, Transcriptomics
Suggests: RefManageR, BiocStyle, knitr, sessioninfo, rmarkdown,
        testthat (>= 3.0.0), TxDb.Scerevisiae.UCSC.sacCer3.sgdGene,
        BSgenome.Scerevisiae.UCSC.sacCer3, GenomeInfoDbData
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/txcutr
git_branch: RELEASE_3_23
git_last_commit: e0101b4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:18:13 UTC; biocbuild
Author: Mervin Fansler [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4108-4218>)
Maintainer: Mervin Fansler <mervin.fansler@bric.ku.dk>
