# Generated by roxygen2: do not edit by hand

export(exportFASTA)
export(exportGTF)
export(exportMergeTable)
export(generateMergeTable)
export(truncateTxome)
export(txdbToGRangesList)
exportMethods(generateMergeTable)
exportMethods(truncateTxome)
importFrom(AnnotationDbi,select)
importFrom(AnnotationDbi,taxonomyId)
importFrom(BiocGenerics,paste)
importFrom(BiocGenerics,which)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpparam)
importFrom(Biostrings,writeXStringSet)
importFrom(GenomicFeatures,exons)
importFrom(GenomicFeatures,exonsBy)
importFrom(GenomicFeatures,extractTranscriptSeqs)
importFrom(GenomicFeatures,genes)
importFrom(GenomicFeatures,transcripts)
importFrom(GenomicRanges,"end<-")
importFrom(GenomicRanges,"start<-")
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,findOverlaps)
importFrom(GenomicRanges,intersect)
importFrom(GenomicRanges,mcols)
importFrom(GenomicRanges,reduce)
importFrom(GenomicRanges,resize)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,start)
importFrom(GenomicRanges,strand)
importFrom(GenomicRanges,width)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,elementNROWS)
importFrom(S4Vectors,expand)
importFrom(S4Vectors,nrun)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(methods,"slot<-")
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,slot)
importFrom(rtracklayer,export)
importFrom(stats,setNames)
importFrom(txdbmaker,makeTxDbFromGRanges)
importFrom(utils,capture.output)
importFrom(utils,write.table)
